'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var pluginutils = require('@rollup/pluginutils');
var path = require('path');
var fs = require('fs');
var libdenolint = require('libdenolint');

const cwd = process.cwd();

function normalizePath(path$1) {
  path$1 = path.relative(cwd, path$1);
  /* c8 ignore next */
  if (path.sep !== '/') path$1 = path$1.split(path.sep).join('/');
  return path$1
}

function denolint({
  include, exclude, configFile = '.denolint.json', ignoreConfig, rules,
  formatter, throwOnWarning = true, throwOnError = true
} = {}) {
  let allRules, excludeRules, includeRules;
  if (rules) {
    ({ all: allRules, exclude: excludeRules, include: includeRules } = rules);
  } else if (!ignoreConfig) {
    try {
      const config = JSON.parse(fs.readFileSync(configFile, 'utf8'));
      const { tags = [], rules = {} } = config;
      if (allRules === undefined) allRules = !tags.includes('recommended')
      ({ exclude: excludeRules, include: includeRules } = rules);
      // eslint-disable-next-line no-empty
    } catch {}
  }
  if (!include) include = ['**/*.js', '**/*.jsx', '**/*.ts', '**/*.tsx'];

  const filter = pluginutils.createFilter(include, exclude || /node_modules/);

  return {
    name: 'denolint',

    async transform(source, id) {
      const path = normalizePath(id);
      if (filter(path)) {
        let warnings;
        try {
          warnings = await libdenolint.lint(path, source, allRules, excludeRules, includeRules);
          if (formatter) warnings = formatter(warnings, id, source);
          for (const warning of warnings) console.warn(warning);
        } catch ({ message }) {
          /* c8 ignore next */
          const suffix = message.includes(path) ? '' : `, at: ${path}`;
          const messages = [`Lint failed: ${message}${suffix}\n`];
          if (formatter) messages = formatter(messages, id, source);
          for (const message of messages) console.error(message);
          if (throwOnError) throw Error('Errors were found')
        }
        if (warnings && warnings.length && throwOnWarning) throw Error('Warnings were found')
      }
      return null
    }
  }
}

function denolintAll({
  include, exclude, configFile = '.denolint.json', ignoreConfig,
  formatter, throwOnWarning = true, throwOnError = true
} = {}) {
  const cwd = process.cwd();

  return {
    name: 'denolint-all',

    async buildStart() {
      let ok;
      try {
        ok = await libdenolint.denolint(cwd, ignoreConfig ? '' : configFile, {
          scanDirs: include, ignorePatterns: exclude
        });
      /* c8 ignore next 6 */
      } catch ({ message }) {
        let messages = [message];
        if (formatter) messages = formatter(messages);
        for (const message of messages) console.error(message);
        if (throwOnError) throw Error('Errors were found')
      }
      if (!ok && throwOnWarning) throw Error('Warnings were found')
    }
  }
}

exports.denolint = denolint;
exports.denolintAll = denolintAll;
//# sourceMappingURL=index.cjs.map
